# -*- coding: utf-8 -*-

"""Aliyun image utils cli module."""

# Copyright (c) 2021 SUSE LLC. All rights reserved.
#
# This file is part of aliyun_img_utils. aliyun_img_utils provides an
# api and command line utilities for handling images in the Aliyun Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import logging
import sys
import click

from aliyun_img_utils.aliyun_image import AliyunImage
from aliyun_img_utils.aliyun_utils import (
    get_config,
    echo_style,
    handle_errors,
    process_shared_options,
    get_logger,
    click_progress_callback
)


shared_options = [
    click.option(
        '-C',
        '--config-dir',
        type=click.Path(exists=True),
        help='Aliyun Image utils config directory to use. Default: '
             '~/.config/aliyun_img_utils/'
    ),
    click.option(
        '--profile',
        help='The configuration profile to use. Expected to match '
             'a config file in config directory. Example: production, '
             'for ~/.config/aliyun_img_utils/production.yaml. The default '
             'value is default: ~/.config/aliyun_img_utils/default.yaml'
    ),
    click.option(
        '--no-color',
        is_flag=True,
        help='Remove ANSI color and styling from output.'
    ),
    click.option(
        '--verbose',
        'log_level',
        flag_value=logging.DEBUG,
        help='Display debug level logging to console.'
    ),
    click.option(
        '--info',
        'log_level',
        flag_value=logging.INFO,
        default=True,
        help='Display logging info to console. (Default)'
    ),
    click.option(
        '--quiet',
        'log_level',
        flag_value=logging.ERROR,
        help='Display only errors to console.'
    ),
    click.option(
        '--access-key',
        type=click.STRING,
        help='Access key used for authentication of requests.'
    ),
    click.option(
        '--access-secret',
        type=click.STRING,
        help='Access secret used for authentication of requests.'
    ),
    click.option(
        '--bucket-name',
        type=click.STRING,
        help='Storage bucket to store uploaded images.'
    ),
    click.option(
        '--region',
        type=click.STRING,
        help='The region to use for the image requests.'
    )
]


def add_options(options):
    def _add_options(func):
        for option in reversed(options):
            func = option(func)
        return func
    return _add_options


def print_license(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo('GPLv3+')
    ctx.exit()


def abort_if_false(ctx, param, value):
    if not value:
        ctx.abort()


@click.group()
@click.version_option()
@click.option(
    '--license',
    is_flag=True,
    callback=print_license,
    expose_value=False,
    is_eager=True,
    help='Show license information.'
)
@click.pass_context
def main(context):
    """
    The command line interface provides aliyun image utilities.

    This includes uploading image tarballs and
    creating/publishing/deprecating framework images.
    """
    if context.obj is None:
        context.obj = {}


@click.group()
def image():
    """
    Image commands.
    """


@click.command()
@click.option(
    '--image-file',
    type=click.Path(exists=True),
    required=True,
    help='Path to qcow2 image.'
)
@click.option(
    '--page-size',
    type=click.IntRange(min=100 * 1024),
    help='Size of page size chunks for image upload. '
         'Minimum chunk size is 100KB.'
)
@click.option(
    '--blob-name',
    type=click.STRING,
    help='Name to use for blob in the storage bucket. By default '
         'the filename from image file will be used.'
)
@click.option(
    '--force-replace-image',
    is_flag=True,
    help='Delete the image prior to upload if it already exists.'
)
@click.option(
    '--timeout',
    type=click.IntRange(min=1),
    default=180,
    help='Session timeout (in minutes) for image upload. '
         'Default is 180 minutes.'
)
@click.option(
    '--direct-transfer',
    'transfer_acceleration',
    flag_value=False,
    help='Upload directly to region endpoint with no acceleration.'
)
@click.option(
    '--accelerated-transfer',
    'transfer_acceleration',
    flag_value=True,
    default=True,
    help='(Default) Use transfer acceleration for image upload. '
         'See docs for more info: '
         'https://www.alibabacloud.com/help/doc-detail/131312.htm.'
)
@add_options(shared_options)
@click.pass_context
def upload(
    context,
    image_file,
    page_size,
    blob_name,
    force_replace_image,
    timeout,
    transfer_acceleration,
    **kwargs
):
    """
    Upload a qcow2 image to a storage bucket in the current region.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger,
            transfer_acceleration=transfer_acceleration,
            timeout=timeout
        )

        keyword_args = {
            'force_replace_image': force_replace_image
        }

        if page_size:
            keyword_args['page_size'] = page_size

        if blob_name:
            keyword_args['blob_name'] = blob_name

        if config_data.log_level != logging.ERROR:
            keyword_args['progress_callback'] = click_progress_callback

        blob_name = aliyun_image.upload_image_tarball(
            image_file,
            **keyword_args
        )

    if config_data.log_level != logging.ERROR:
        echo_style(
            f'Image uploaded as {blob_name}',
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    required=True,
    help='Name of the newly created compute image.'
)
@click.option(
    '--image-description',
    type=click.STRING,
    required=True,
    help='Description for the newly created image.'
)
@click.option(
    '--platform',
    type=click.STRING,
    required=True,
    help='The distribution of the image operating system.'
)
@click.option(
    '--blob-name',
    type=click.STRING,
    required=True,
    help='Name for the blob in the storage bucket to use '
         'to create the new image.'
)
@click.option(
    '--force-replace-image',
    is_flag=True,
    help='Delete the compute image prior to creation if it already exists.'
)
@click.option(
    '--disk-size',
    type=click.IntRange(min=5),
    help='Size root disk in GB. Default is 20GB.'
)
@add_options(shared_options)
@click.pass_context
def create(
    context,
    image_name,
    image_description,
    platform,
    blob_name,
    force_replace_image,
    disk_size,
    **kwargs
):
    """
    Create a compute image from a qcow2 image in storage.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        keyword_args = {
            'force_replace_image': force_replace_image
        }

        if disk_size:
            keyword_args['disk_image_size'] = disk_size

        image_id = aliyun_image.create_compute_image(
            image_name,
            image_description,
            blob_name,
            platform,
            **keyword_args
        )

    if config_data.log_level != logging.ERROR:
        echo_style(
            f'Image created with id: {image_id}',
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image to be deleted.',
    required=True
)
@click.option(
    '--delete-blob',
    is_flag=True,
    help='Also delete the image blob from storage bucket.'
)
@click.option(
    '--force',
    is_flag=True,
    help='Forcibly deletes the custom image, regardless of '
         'whether the image is being used by other instances.'
)
@click.option(
    '--regions',
    help='A comma separated list of region ids for '
         'deleting the provided image. If no regions '
         'are provided the image will be deleted in all '
         'available regions.'
)
@add_options(shared_options)
@click.pass_context
def delete(context, image_name, delete_blob, force, regions, **kwargs):
    """Delete a compute image and optionally the backing qcow2 blob."""
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        keyword_args = {
            'delete_blob': delete_blob,
            'force': force
        }

        if regions:
            regions = regions.split(',')
            keyword_args['regions'] = regions

        if click.confirm(f'Are you sure you want to delete {image_name}'):
            aliyun_image.delete_compute_image_in_regions(
                image_name,
                **keyword_args
            )
        else:
            sys.exit(0)


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image to be copied.',
    required=True
)
@click.option(
    '--regions',
    help='A comma separated list of region ids to '
         'copy the provided image to. If no regions '
         'are provided the image will be copied to all '
         'available regions.'
)
@add_options(shared_options)
@click.pass_context
def replicate(context, image_name, regions, **kwargs):
    """
    Replicate a compute image to a set of regions.

    If no regions are provided the image is replicated to all
    available regions.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        keyword_args = {}

        if regions:
            regions = regions.split(',')
            keyword_args['regions'] = regions

        images = aliyun_image.replicate_image(image_name, **keyword_args)

    if config_data.log_level != logging.ERROR:
        echo_style(
            json.dumps(images, indent=2),
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image to be published.',
    required=True
)
@click.option(
    '--launch-permission',
    type=click.STRING,
    help='The launch permission to set for the published image.',
    required=True
)
@click.option(
    '--regions',
    help='A comma separated list of region ids to '
         'publish the provided image in. If no regions '
         'are provided the image will be published in all '
         'available regions.'
)
@add_options(shared_options)
@click.pass_context
def publish(context, image_name, launch_permission, regions, **kwargs):
    """
    Publish a compute image in a set of regions.

    If no regions are provided the image is published to all
    available regions.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        keyword_args = {}

        if regions:
            regions = regions.split(',')
            keyword_args['regions'] = regions

        aliyun_image.publish_image_to_regions(
            image_name,
            launch_permission,
            **keyword_args
        )

    if config_data.log_level != logging.ERROR:
        echo_style(
            f'Image published: {image_name}',
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image to be deprecated.',
    required=True
)
@click.option(
    '--regions',
    help='A comma separated list of region ids to '
         'deprecate the provided image in. If no regions '
         'are provided the image will be deprecated in all '
         'available regions.'
)
@click.option(
    '--replacement-image',
    type=click.STRING,
    help='Name of the replacement image.',
)
@click.option(
    '--deprecation-period',
    type=click.INT,
    default=6,
    help='Period in months the image will be deprecated before deletion.',
)
@add_options(shared_options)
@click.pass_context
def deprecate(
    context,
    image_name,
    regions,
    replacement_image,
    deprecation_period,
    **kwargs
):
    """
    Deprecate compute in a set of regions.

    If no regions are provided the image is deprecated in all
    available regions.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger,
            deprecation_period=deprecation_period
        )

        keyword_args = {}

        if regions:
            regions = regions.split(',')
            keyword_args['regions'] = regions

        if replacement_image:
            keyword_args['replacement_image'] = replacement_image

        aliyun_image.deprecate_image_in_regions(
            image_name,
            **keyword_args
        )

    if config_data.log_level != logging.ERROR:
        echo_style(
            f'Image deprecated: {image_name}',
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image to be activated.',
    required=True
)
@click.option(
    '--regions',
    help='A comma separated list of region ids to '
         'activate the provided image in. If no regions '
         'are provided the image will be activated in all '
         'available regions.'
)
@add_options(shared_options)
@click.pass_context
def activate(context, image_name, regions, **kwargs):
    """
    Activate compute image (make available) in a set of regions.

    If no regions are provided the image is activated in all
    available regions.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        keyword_args = {}

        if regions:
            regions = regions.split(',')
            keyword_args['regions'] = regions

        aliyun_image.activate_image_in_regions(image_name, **keyword_args)

    if config_data.log_level != logging.ERROR:
        echo_style(
            f'Image activated: {image_name}',
            config_data.no_color
        )


@click.command()
@click.option(
    '--image-name',
    type=click.STRING,
    help='Name of the image.'
)
@click.option(
    '--image-id',
    type=click.STRING,
    help='ID of the image.'
)
@click.option(
    '--deprecated',
    is_flag=True,
    help='If set the search is filtered on images '
         'in deprecated state.'
)
@add_options(shared_options)
@click.pass_context
def info(context, image_name, image_id, deprecated, **kwargs):
    """
    Get a dictionary of image data for an image based on ID or name.

    If `--deprecated` all images in deprecated state are searched
    otherwise all images in active state are searched.
    """
    process_shared_options(context.obj, kwargs)
    config_data = get_config(context.obj)
    logger = get_logger(config_data.log_level)

    with handle_errors(config_data.log_level, config_data.no_color):
        aliyun_image = AliyunImage(
            config_data.access_key,
            config_data.access_secret,
            config_data.region,
            config_data.bucket_name,
            log_level=config_data.log_level,
            log_callback=logger
        )

        image_data = aliyun_image.get_compute_image(
            image_name=image_name,
            image_id=image_id,
            is_deprecated=deprecated
        )

    echo_style(
        json.dumps(image_data, indent=2),
        config_data.no_color
    )


image.add_command(activate)
image.add_command(create)
image.add_command(delete)
image.add_command(deprecate)
image.add_command(publish)
image.add_command(replicate)
image.add_command(upload)
image.add_command(info)
main.add_command(image)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""Aliyun img utils class tests."""

# Copyright (c) 2021 SUSE LLC. All rights reserved.
#
# This file is part of aliyun_img_utils. aliyun_img_utils provides an
# api and command line utilities for handling images in the Aliyun Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import oss2

from unittest.mock import patch, Mock

from pytest import raises

from aliyun_img_utils.aliyun_image import AliyunImage
from aliyun_img_utils.aliyun_exceptions import (
    AliyunException,
    AliyunImageException,
    AliyunImageCreateException,
    AliyunImageUploadException
)


class TestAliyunImage(object):
    """Test Aliyun Image class."""

    def setup(self):
        self.image = AliyunImage(
            '12345',
            '54321',
            'cn-beijing',
            bucket_name='test-bucket'
        )

    @patch('aliyun_img_utils.aliyun_image.get_storage_auth')
    @patch('aliyun_img_utils.aliyun_image.get_storage_bucket_client')
    def test_bucket_client(self, mock_bucket_client, mock_storage_auth):
        client = Mock()
        mock_bucket_client.return_value = client
        assert self.image.bucket_client

        # Server Error
        client.get_bucket_info.side_effect = oss2.exceptions.ServerError(
            'Failed', Mock(), Mock(), {'Message': 'Failed'}
        )
        self.image._bucket_client = None

        with raises(AliyunException):
            assert self.image.bucket_client

        # Request Error
        client.get_bucket_info.side_effect = oss2.exceptions.RequestError(
            'Failed'
        )
        self.image._bucket_client = None

        with raises(AliyunException):
            assert self.image.bucket_client

        # Exception
        client.get_bucket_info.side_effect = Exception('Failed')
        self.image._bucket_client = None

        with raises(AliyunException):
            assert self.image.bucket_client

    def test_image_tarball_exists(self):
        client = Mock()
        self.image._bucket_client = client
        assert self.image.image_tarball_exists('blob.qcow2')

        # Not exists
        client.get_object_meta.side_effect = oss2.exceptions.NoSuchKey(
            {}, {}, {}, {}
        )
        assert self.image.image_tarball_exists('blob.qcow2') is False

    @patch.object(AliyunImage, 'image_tarball_exists')
    def test_wait_on_blob(self, mock_image_tarball_exists):
        mock_image_tarball_exists.return_value = True
        self.image.wait_on_blob('blob.qcow2')

    def test_delete_image_tarball(self):
        client = Mock()
        response = Mock()
        response.status = 200
        client.delete_object.return_value = response
        self.image._bucket_client = client

        assert self.image.delete_storage_blob('blob.qcow2')

        # Not exists
        response.status = 204
        assert self.image.delete_storage_blob('blob.qcow2') is False

    @patch.object(AliyunImage, 'wait_on_blob')
    @patch('aliyun_img_utils.aliyun_image.put_blob')
    def test_upload_image_tarball(self, mock_put_blob, mock_wait_on_blob):
        callback = Mock()
        client = Mock()
        self.image._bucket_client = client

        assert self.image.upload_image_tarball(
            'tests/data/blob.qcow2',
            page_size=8 * 8 * 1024,
            progress_callback=callback,
            force_replace_image=True
        )

        # File Not Found
        mock_put_blob.side_effect = FileNotFoundError('Not there!')
        with raises(AliyunImageUploadException):
            self.image.upload_image_tarball(
                'tests/data/blob.qcow2',
                page_size=8 * 8 * 1024,
                progress_callback=callback,
                force_replace_image=True
            )

        # Server Error
        mock_put_blob.side_effect = oss2.exceptions.ServerError(
            'Failed', {}, {}, {'Message': 'Failed'}
        )
        with raises(AliyunImageUploadException):
            self.image.upload_image_tarball(
                'tests/data/blob.qcow2',
                page_size=8 * 8 * 1024,
                progress_callback=callback,
                force_replace_image=True
            )

        # Request Error
        mock_put_blob.side_effect = oss2.exceptions.RequestError(
            'Failed'
        )
        with raises(AliyunImageUploadException):
            self.image.upload_image_tarball(
                'tests/data/blob.qcow2',
                page_size=8 * 8 * 1024,
                progress_callback=callback,
                force_replace_image=True
            )

        # Request Error
        mock_put_blob.side_effect = Exception('Failed')
        with raises(AliyunImageUploadException):
            self.image.upload_image_tarball(
                'tests/data/blob.qcow2',
                page_size=8 * 8 * 1024,
                progress_callback=callback,
                force_replace_image=True
            )

    @patch.object(AliyunImage, 'delete_storage_blob')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_delete_compute_image(self, mock_get_image, mock_delete_tarball):
        image = {
            'ImageId': 'm-123',
            'DiskDeviceMappings': {
                'DiskDeviceMapping': [{'ImportOSSObject': 'test-blob'}]
            }
        }
        mock_get_image.side_effect = [
            image,
            AliyunImageException,
            AliyunImageException
        ]

        client = Mock()
        self.image._compute_client = client

        assert self.image.delete_compute_image(
            'test-image',
            delete_blob=True,
            force=True
        )

        # Image not exists
        assert self.image.delete_compute_image(
            'test-image',
            delete_blob=True
        ) is False

    @patch.object(AliyunImage, 'delete_compute_image')
    @patch.object(AliyunImage, 'get_regions')
    def test_delete_compute_image_in_regions(
        self,
        mock_get_regions,
        mock_delete_image
    ):
        mock_get_regions.return_value = ['cn-beijing']
        self.image.delete_compute_image_in_regions('test-image')

    def test_compute_image_exists(self):
        response = json.dumps({'Images': {'Image': [{'image1': 'info'}]}})
        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        assert self.image.image_exists('test-image')

        # Not exists
        client.do_action_with_exception.side_effect = Exception
        assert self.image.image_exists('test-image') is False

    @patch.object(AliyunImage, 'get_compute_image')
    def test_create_compute_image(self, mock_get_image):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        result = self.image.create_compute_image(
            'test-image',
            'test description',
            'test-blob.qcow2',
            'SLES'
        )
        assert result == 'm-123'

        # Create failure
        client.do_action_with_exception.return_value = Exception
        with raises(AliyunImageCreateException):
            self.image.create_compute_image(
                'test-image',
                'test description',
                'test-blob.qcow2',
                'SLES'
            )

    def test_get_regions(self):
        response = json.dumps({
            'Regions': {'Region': [{'RegionId': 'cn-beijing'}]}
        })
        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        regions = self.image.get_regions()
        assert 'cn-beijing' in regions

        # Failed to get regions
        client.do_action_with_exception.return_value = Exception

        with raises(AliyunException):
            self.image.get_regions()

    def test_bucket_name_var(self):
        client = Mock()
        self.image._bucket_client = client

        self.image.bucket_name = 'bucket2'
        assert self.image._bucket_client is None

    def test_region_var(self):
        client = Mock()
        self.image._bucket_client = client
        self.image._compute_client = client

        self.image.region = 'cn-beijing'
        assert self.image._bucket_client is None
        assert self.image._compute_client is None

    @patch.object(AliyunImage, 'get_regions')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_replicate_image(self, mock_get_image, mock_get_regions):
        image = {'ImageId': 'm-123', 'Description': 'Test image'}
        response = json.dumps(image)
        mock_get_image.return_value = image
        mock_get_regions.return_value = ['cn-shanghai']

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.replicate_image('test-image')

        # Replicate failure
        client.do_action_with_exception.side_effect = Exception
        self.image.replicate_image('test-image')

    @patch.object(AliyunImage, 'publish_image')
    @patch.object(AliyunImage, 'get_regions')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_publish_image_to_regions(
        self,
        mock_get_image,
        mock_get_regions,
        mock_publish_image
    ):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image
        mock_get_regions.return_value = ['cn-beijing']

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.publish_image_to_regions('test-image', 'VISIBLE')

    @patch.object(AliyunImage, 'get_compute_image')
    def test_publish_image(self, mock_get_image):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.publish_image('test-image', 'VISIBLE')

        # Publish failure
        client.do_action_with_exception.side_effect = Exception
        with raises(AliyunImageException):
            self.image.publish_image('test-image', 'VISIBLE')

    @patch.object(AliyunImage, 'deprecate_image')
    @patch.object(AliyunImage, 'get_regions')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_deprecate_image_in_regions(
        self,
        mock_get_image,
        mock_get_regions,
        mock_deprecate_image
    ):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image
        mock_get_regions.return_value = ['cn-beijing']

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.deprecate_image_in_regions('test-image')

    @patch.object(AliyunImage, 'add_image_tags')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_deprecate_image(
        self,
        mock_get_image,
        mock_add_tags
    ):
        image = {'ImageId': 'm-123'}
        mock_get_image.return_value = image

        self.image.deprecate_image('test-image')
        assert mock_add_tags.call_count == 1

    @patch.object(AliyunImage, 'activate_image')
    @patch.object(AliyunImage, 'get_regions')
    @patch.object(AliyunImage, 'get_compute_image')
    def test_activate_image_in_regions(
        self,
        mock_get_image,
        mock_get_regions,
        mock_activate_image
    ):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image
        mock_get_regions.return_value = ['cn-beijing']

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.activate_image_in_regions('test-image')

    @patch.object(AliyunImage, 'get_compute_image')
    def test_activate_image(self, mock_get_image):
        image = {'ImageId': 'm-123'}
        response = json.dumps(image)
        mock_get_image.return_value = image

        client = Mock()
        client.do_action_with_exception.return_value = response
        self.image._compute_client = client

        self.image.activate_image('test-image')

        # Activate failure
        client.do_action_with_exception.side_effect = Exception
        with raises(AliyunImageException):
            self.image.activate_image('test-image')

    def test_add_tags_to_image(self):
        tags = [{'Replacement image': 'i-123456'}]

        client = Mock()
        self.image._compute_client = client

        self.image.add_image_tags('m-123', tags)

        # Activate failure
        client.do_action_with_exception.side_effect = Exception
        with raises(AliyunImageException):
            self.image.add_image_tags('m-123', tags)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""Aliyun img utils tests."""

# Copyright (c) 2021 SUSE LLC. All rights reserved.
#
# This file is part of aliyun_img_utils. aliyun_img_utils provides an
# api and command line utilities for handling images in the Aliyun Cloud.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from aliyun_img_utils.aliyun_exceptions import AliyunException
from pytest import raises
from unittest.mock import patch, Mock

from aliyun_img_utils.aliyun_utils import (
    put_blob,
    click_progress_callback,
    get_compute_client,
    import_key_pair,
    delete_key_pair
)


@patch('aliyun_img_utils.aliyun_utils.click')
def tests_click_progress_bar(mock_click):
    bar = Mock()
    bar.pos = 0
    mock_click.progressbar.return_value = bar
    click_progress_callback(15, 15)


@patch('aliyun_img_utils.aliyun_utils.oss2')
def test_put_blob(mock_oss2):
    client = Mock()
    callback = Mock()

    mock_oss2.determine_part_size.return_value = 15

    put_blob(
        client,
        'blob.vhd',
        'tests/data/blob.vhd',
        progress_callback=callback
    )


@patch('aliyun_img_utils.aliyun_utils.AcsClient')
def test_get_compute_client(mock_acs):
    client = Mock()
    mock_acs.return_value = client

    result = get_compute_client('123', '321', 'cn-beijing')
    assert result == client

    # Exception
    mock_acs.side_effect = Exception('Invalid credentials!')

    with raises(AliyunException):
        get_compute_client('123', '321', 'cn-beijing')


def test_import_key():
    client = Mock()
    import_key_pair('key123', '321', client)
    assert client.do_action_with_exception.call_count == 1

    # Exception
    client.do_action_with_exception.side_effect = Exception('Duplicate key!')

    with raises(AliyunException):
        import_key_pair('key123', '321', client)


def test_delete_key():
    client = Mock()
    delete_key_pair('key123', client)
    assert client.do_action_with_exception.call_count == 1

    # Exception
    client.do_action_with_exception.side_effect = Exception('Invalid key!')

    with raises(AliyunException):
        delete_key_pair('key123', client)
